/* This is a C source file #included at global scope in status.c. There are a
 * number of useful functions defined in status.c and #included headers to try
 * to make iteration a little more convenient. */

/* Environment
 * getenv(key) gets the value of an environment variable
 * setenv(key, val, overwrite) sets key to val in the environment, overwriting
 *	val if key was already defined and overwrite is 0
 * set_timezone(val) sets TZ to val in the environment, then calls tzset(3) to
 *	set appropriate timezone values. if val is NULL, it clears the TZ val,
 *	which restores the timezone to the system local time
 * get(key) gets the val corresponding to key in settings
 * set(key, val) sets the val corresponding to key in settings
 * idx(key) returns the index of key in settings
 * del(key) deletes the first instance of key in settings
 * cpy(key) creates an identical instance to key in settings
 * ren(key, dst) renames key to dst in settings
 * keyval_init() cleans the settings in case of a glitch */

/* I/O
 * out(s) holds s and prints what it was previously holding, if it wasn't NULL
 * drop() sets out()'s held value to NULL */

/* Modules
 * mod_static(s) outputs s and then the separator val, unless mod_static() is
 *	called as the last function in order
 * mod_time(fmt) outputs the time for the current time zone in the strftime(3)
 *	fmt */
void
order() {
	/* there are a couple good ways to print without the separator. the
	 * best is to use out() to write directly to the output buffer */
	out("utc"); /* => "utc" */
	/* but another way is to use status_static to write a message and
	 * separator, then use drop() to recall the last thing written (always
	 * the separator in status_* functions) */
	mod_static(" "); drop(); /* => " " */
	set_timezone("");
	mod_time("%Y-%m-%dT%H:%M"); /* => "2025-03-27T00:00 // " */

	/* and another way is to use the weird key-val system */
	cpy("separator");      /* duplicate setting */
	set("separator", " "); /* overwrite first setting */
	set_timezone(NULL);
	mod_static("local"); /* use first setting */
	del("separator");       /* delete the first, restoring the second */
	mod_time("%Y-%m-%dT%H:%M");
	drop();
	/* => "local 2025-03-26T18:00" */

	/* all => "utc 2025-03-27T00:00 // local 2025-03-26T18:00" */
}
